<?php

/**
 * Class Frontend.
 */
class frontend
{
    /**
     * @var SOAPAuth
     */
    private $api;

    /**
     *
     */
    public function __construct()
    {
        $this->api = new SOAPAuth(get_option('cl_wp_pl_user_id'), get_option('cl_wp_pl_api_key'), '1.0');

        add_action('wp_register_script', array(
                $this,
                'cewp_add_css_js',
        ));
        // if not connected disable all frontend functionality
        //@TODO show all options only for admin if it is in test mode
        if (!$this->api->cewp_is_connected()) {
            return false;
        }

        //add styles and javascript
        add_action('wp_enqueue_scripts', array(
                $this,
                'cewp_add_css_js',
        ));

        add_action('wp_ajax_nopriv_my_ajax', array(
                $this,
                'cewp_ajax_callback',
        ));
        add_action('wp_ajax_my_ajax', array(
                $this,
                'cewp_ajax_callback',
        ));

        if (isset($_GET['action']) and $_GET['action'] == 'register') {
            if (get_option('cl_wp_pl_api_to_registration') == 1) {
                add_filter('register_form', array(
                        $this,
                        'cewp_add_subscribe_field',
                ));
                add_action('user_register', array(
                        $this,
                        'cewp_send_subscribe_checkbox',
                ));
            }
        } else {
            if (get_option('cl_wp_pl_api_to_comment') == 1) {
                add_filter('comment_form_field_comment', array(
                        $this,
                        'cewp_add_subscribe_field',
                ));
                add_action('comment_post', array(
                        $this,
                        'cewp_send_subscribe_checkbox',
                ));
            }
        }

        add_filter('the_content', array($this, 'cewp_replace_content'));
        add_filter('widget_text', array($this, 'cewp_replace_content'));
    }

    /**
     * @param $content
     *
     * @return mixed
     */
    public function cewp_replace_content($content)
    {
        $form_content = new cl_wp_pl_widget();
        $form_content = $form_content->cewp_form_view();

        $content = str_replace('[CE_FORM_CODE]', $form_content, $content);

        return $content;
    }

    /**
     * Add checkbox for subscription.
     *
     * @param $fields current comment fields
     *
     * @return string
     */
    public function cewp_add_subscribe_field($fields)
    {
        $value = ($fields) ? 1 : 2;

        $checked = (isset($_POST['cl_wp_pl_api_register_to_api_checkbox']) and $_POST['cl_wp_pl_api_register_to_api_checkbox'] == 2) ? 'checked="checked"' : '';

        $field_html = '';
        $field_html .= '<div class="cl_wp_pl_subscribe_checkbox_row">';
        $field_html .= '<input type = "checkbox" id = "cl_wp_pl_api_register_to_api_checkbox" name="cl_wp_pl_api_register_to_api_checkbox" value="'.$value.'" '.$checked.'>';
        $field_html .= '<label for="cl_wp_pl_api_register_to_api_checkbox" >'.get_option('cl_wp_pl_checkbox_title').'</label>';
        $field_html .= '<div class="clear clearfix"></div>';
        $field_html .= '</div>';

        if ($fields) {
            $fields .= $field_html;

            return $fields;
        }

        echo $field_html;
    }

    /**
     *  save subscriber.
     */
    public function cewp_send_subscribe_checkbox($from)
    {
        $checkbox_value = $_POST['cl_wp_pl_api_register_to_api_checkbox'];

        if (isset($checkbox_value) and ($checkbox_value == 1 or $checkbox_value == 2)) {
            $user = $this->cewp_get_user_data($_POST, $checkbox_value);

            $this->api->cewp_subscribe_to_all($user);
        }
    }

    /**
     * Get information of the user that is send to Clever Elements system
     * It depends if user is registered or not
     * Function is used for both login and comment forms.
     *
     * @param null $post
     * @param int  $from
     *
     * @return array
     */
    public function cewp_get_user_data($post = null, $from = 1)
    {
        // default = 1 (1 - from comments form, 2- from registration form)
        if ($from == 1) {
            $current_user = wp_get_current_user();
            $user = array();

            // not registered user
            if (!$current_user->ID) {
                //set email
                $user['email'] = $post['email'];
                //set name
                $user['name'] = $post['author'];
            } else {
                // get registered user
                //set email
                $user['email'] = $current_user->user_email;

                //set name
                $user['name'] = $current_user->display_name;

                // if first or last name are set
                if (isset($user['first_name']) or isset($user['last_name'])) {
                    $user['name'] = trim($current_user->user_firstname.' '.$current_user->user_lastname);
                }
            }
        }

        // registration form
        if ($from == 2) {
            $user['name'] = $post['user_login'];
            $user['email'] = $post['user_email'];
        }

        return $user;
    }

    /**
     * add js and css.
     */
    public function cewp_add_css_js()
    {

        // add css and js
        wp_enqueue_style('cl-style', C_L_PUBLIC_URL.'css/front_end.css', array(), '1.0.0', false);
        wp_enqueue_script('cl-frontend', C_L_PUBLIC_URL.'js/front_end.js', array('jquery'));
        wp_localize_script('cl-frontend', 'ajax_object', array('ajax_url' => admin_url('admin-ajax.php')));

        return true;
    }

    /**
     *
     */
    public function cewp_ajax_callback()
    {
        // get post string from serialized form
        parse_str($_POST['data'], $post);

        $post_user_data['name'] = isset($post['name']) ? $post['name'] : '';
        $post_user_data['email'] = $post['email'];

        unset($post['name']);
        unset($post['email']);

        $this->api->cewp_subscribe_to_form($post_user_data, $post);
        $errors = $this->api->cewp_get_errors();

        // no errors
        if (count($errors) == 0) {
            echo json_encode(array('status' => 1));
            exit();
        } else {
            // no errors
            echo json_encode(array('status' => 0));
            exit();
        }
    }
}

// call class
$front_end = new Frontend();
